@@ -241,10 +241,11 @@ typedef float             float32_t;
  *   Used in findlib, filed, and plugins
  */
 enum {
-   CF_SKIP = 1,                       /* skip file (not newer or something) */
-   CF_ERROR,                          /* error creating file */
-   CF_EXTRACT,                        /* file created, data to extract */
-   CF_CREATED                         /* file created, no data to extract */
+   CF_SKIP = 1,                   /* skip file (not newer or something) */
+   CF_ERROR,                      /* error creating file */
+   CF_EXTRACT,                    /* file created, data to extract */
+   CF_CREATED,                    /* file created, no data to extract */
+   CF_CORE                        /* let bacula core handle the file creation */
 };
 
 #ifndef MAX
