@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2008 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -26,22 +26,15 @@
    Switzerland, email:ftf@fsfeurope.org.
 */
 
+extern const char CATS_IMP_EXP *get_restore_objects;
 extern const char CATS_IMP_EXP *fill_jobhisto;
 extern const char CATS_IMP_EXP *client_backups;
 extern const char CATS_IMP_EXP *list_pool;
 extern const char CATS_IMP_EXP *drop_deltabs[];
 extern const char CATS_IMP_EXP *create_delindex;
-extern const char CATS_IMP_EXP *select_job;
-extern const char CATS_IMP_EXP *count_select_job;
-extern const char CATS_IMP_EXP *del_File;
 extern const char CATS_IMP_EXP *cnt_File;
-extern const char CATS_IMP_EXP *cnt_DelCand;
-extern const char CATS_IMP_EXP *del_Job;
 extern const char CATS_IMP_EXP *del_MAC;
-extern const char CATS_IMP_EXP *del_JobMedia;
-extern const char CATS_IMP_EXP *cnt_JobMedia;
 extern const char CATS_IMP_EXP *sel_JobMedia;
-extern const char CATS_IMP_EXP *upd_Purged;
 
 extern const char CATS_IMP_EXP *cleanup_created_job;
 extern const char CATS_IMP_EXP *cleanup_running_job;
@@ -57,6 +50,7 @@ extern const char CATS_IMP_EXP *uar_inc;
 extern const char CATS_IMP_EXP *uar_list_temp;
 extern const char CATS_IMP_EXP *uar_sel_all_temp1;
 extern const char CATS_IMP_EXP *uar_sel_fileset;
+extern const char CATS_IMP_EXP *uar_sel_filesetid;
 extern const char CATS_IMP_EXP *uar_mediatype;
 extern const char CATS_IMP_EXP *uar_jobid_fileindex;
 extern const char CATS_IMP_EXP *uar_dif;
@@ -66,13 +60,27 @@ extern const char CATS_IMP_EXP *uar_jobids_fileindex;
 extern const char CATS_IMP_EXP *uar_jobid_fileindex_from_table;
 extern const char CATS_IMP_EXP *uar_sel_jobid_temp;
 
-extern const char CATS_IMP_EXP *select_recent_version[4];
-extern const char CATS_IMP_EXP *select_recent_version_with_basejob[4];
-extern const char CATS_IMP_EXP *create_deltabs[5];
+extern const char CATS_IMP_EXP *select_recent_version[];
+extern const char CATS_IMP_EXP *select_recent_version_with_basejob[];
+extern const char CATS_IMP_EXP *select_recent_version_with_basejob_and_delta[];
+extern const char CATS_IMP_EXP *create_temp_accurate_jobids[];
+extern const char CATS_IMP_EXP *create_temp_basefile[];
+extern const char CATS_IMP_EXP *create_temp_new_basefile[];
+extern const char CATS_IMP_EXP *create_deltabs[];
+extern const char CATS_IMP_EXP *uap_upgrade_copies_oldest_job[];
 
-extern const char CATS_IMP_EXP *uar_file[4];
-extern const char CATS_IMP_EXP *uar_create_temp[5];
-extern const char CATS_IMP_EXP *uar_create_temp1[5];
-extern const char CATS_IMP_EXP *uar_jobid_fileindex_from_dir[4];
-extern const char CATS_IMP_EXP *sql_get_max_connections[4];
-extern const uint32_t CATS_IMP_EXP sql_get_max_connections_index[4];
+extern const char CATS_IMP_EXP *uar_file[];
+extern const char CATS_IMP_EXP *uar_create_temp[];
+extern const char CATS_IMP_EXP *uar_create_temp1[];
+extern const char CATS_IMP_EXP *uar_jobid_fileindex_from_dir[];
+extern const char CATS_IMP_EXP *sql_media_order_most_recently_written[];
+extern const char CATS_IMP_EXP *sql_get_max_connections[];
+extern const char CATS_IMP_EXP *sql_bvfs_select[];
+extern const char CATS_IMP_EXP *sql_bvfs_list_files[];
+
+extern const char CATS_IMP_EXP *batch_lock_path_query[];
+extern const char CATS_IMP_EXP *batch_lock_filename_query[];
+extern const char CATS_IMP_EXP *batch_unlock_tables_query[];
+extern const char CATS_IMP_EXP *batch_fill_path_query[];
+extern const char CATS_IMP_EXP *batch_fill_filename_query[];
+extern const char CATS_IMP_EXP *match_query[];
