@@ -123,7 +123,7 @@ static struct cmdstruct commands[] = { /* help */  /* can be used in runscript *
  { NT_(".bvfs_cleanup"), dot_bvfs_cleanup,       NULL,       true},
  { NT_(".bvfs_clear_cache"),dot_bvfs_clear_cache,NULL,       false},
  { NT_(".types"),      typescmd,                 NULL,       false}
-             };
+};
 #define comsize ((int)(sizeof(commands)/sizeof(struct cmdstruct)))
 
 /*
@@ -207,7 +207,10 @@ static bool dot_bvfs_clear_cache(UAContext *ua, const char *cmd)
       Bvfs fs(ua->jcr, ua->db);
       fs.clear_cache();
       ua->info_msg("OK\n");
-   } 
+   } else {
+      ua->error_msg("Can't find 'yes' argument\n");
+   }
+
    close_db(ua);
    return true;
 }
@@ -542,6 +545,12 @@ static bool dot_bvfs_get_jobids(UAContext *ua, const char *cmd)
       return true;
    }
 
+   /* When in level base, we don't rely on any Full/Incr/Diff */
+   if (jr.JobLevel == L_BASE) {
+      ua->send_msg("%s\n", edit_int64(jr.JobId, ed1));
+      return true;
+   }
+
    /* If we have the "all" option, we do a search on all defined fileset
     * for this client
     */
