@@ -49,7 +49,7 @@ void stop_dir_heartbeat(JCR *jcr);
 
 /* From acl.c */
 bacl_exit_code build_acl_streams(JCR *jcr, FF_PKT *ff_pkt);
-bacl_exit_code parse_acl_streams(JCR *jcr, int stream);
+bacl_exit_code parse_acl_streams(JCR *jcr, int stream, char *content, uint32_t content_length);
 
 /* from accurate.c */
 bool accurate_finish(JCR *jcr);
@@ -64,9 +64,15 @@ void unstrip_path(FF_PKT *ff_pkt);
 
 /* from xattr.c */
 bxattr_exit_code build_xattr_streams(JCR *jcr, FF_PKT *ff_pkt);
-bxattr_exit_code parse_xattr_streams(JCR *jcr, int stream);
+bxattr_exit_code parse_xattr_streams(JCR *jcr, int stream, char *content, uint32_t content_length);
 
 /* from job.c */
-findFILESET *new_exclude(JCR *jcr);
-void add_file_to_fileset(JCR *jcr, const char *fname, findFILESET *fileset,
-                         bool is_file); 
+findINCEXE *new_exclude(JCR *jcr);
+findINCEXE *new_preinclude(JCR *jcr);
+void set_incexe(JCR *jcr, findINCEXE *incexe);
+void new_options(JCR *jcr, findINCEXE *incexe);
+void add_file_to_fileset(JCR *jcr, const char *fname, bool is_file); 
+int add_options_to_fileset(JCR *jcr, const char *item);
+int add_wild_to_fileset(JCR *jcr, const char *item, int type);
+int add_regex_to_fileset(JCR *jcr, const char *item, int type);
+findINCEXE *new_include(JCR *jcr);
