@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2008 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -28,13 +28,12 @@
 /*
  * Prototypes for finlib directory of Bacula
  *
- *   Version $Id$
  */
 
 /* from attribs.c */
-void    encode_stat       (char *buf, struct stat *statp, int32_t LinkFI, int data_stream);
-int     decode_stat       (char *buf, struct stat *statp, int32_t *LinkFI);
-int32_t decode_LinkFI     (char *buf, struct stat *statp);
+void    encode_stat       (char *buf, struct stat *statp, int stat_size, int32_t LinkFI, int data_stream);
+int     decode_stat       (char *buf, struct stat *statp, int stat_size, int32_t *LinkFI);
+int32_t decode_LinkFI     (char *buf, struct stat *statp, int stat_size);
 int     encode_attribsEx  (JCR *jcr, char *attribsEx, FF_PKT *ff_pkt);
 bool    set_attributes    (JCR *jcr, ATTR *attr, BFILE *ofd);
 int     select_data_stream(FF_PKT *ff_pkt);
@@ -69,6 +68,9 @@ int   find_one_file(JCR *jcr, FF_PKT *ff,
                char *p, dev_t parent_device, bool top_level);
 int   term_find_one(FF_PKT *ff);
 bool  has_file_changed(JCR *jcr, FF_PKT *ff_pkt);
+bool check_changes(JCR *jcr, FF_PKT *ff_pkt);
+void ff_pkt_set_link_digest(FF_PKT *ff_pkt, 
+                            int32_t digest_stream, const char *digest, uint32_t len);
 
 /* From get_priv.c */
 int enable_backup_privileges(JCR *jcr, int ignore_errors);
@@ -78,9 +80,14 @@ int enable_backup_privileges(JCR *jcr, int ignore_errors);
 bool makepath(ATTR *attr, const char *path, mode_t mode,
            mode_t parent_mode, uid_t owner, gid_t group,
            int keep_dir_modes);
+void free_path_list(JCR *jcr);
+bool path_list_lookup(JCR *jcr, char *fname);
+bool path_list_add(JCR *jcr, uint32_t len, char *fname);
+
 
 /* from fstype.c */
 bool fstype(const char *fname, char *fs, int fslen);
+bool fstype_equals(const char *fname, const char *fstypename);
 
 /* from drivetype.c */
 bool drivetype(const char *fname, char *fs, int fslen);
