@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -145,13 +145,14 @@ void e_msg(const char *file, int line, int type, int level, const char *fmt,...)
 void Jmsg(JCR *jcr, int type, utime_t mtime, const char *fmt,...);
 void Qmsg(JCR *jcr, int type, utime_t mtime, const char *fmt,...);
 bool get_trace(void);
+const char *get_basename(const char *pathname);
 
-struct B_DB;
-typedef void (*sql_query)(JCR *jcr, const char *cmd);
-typedef void (*sql_escape)(JCR *jcr, B_DB* db, char *snew, char *old, int len);
+class B_DB;
+typedef void (*sql_query_func)(JCR *jcr, const char *cmd);
+typedef void (*sql_escape_func)(JCR *jcr, B_DB* db, char *snew, char *old, int len);
 
-extern DLL_IMP_EXP sql_query     p_sql_query;
-extern DLL_IMP_EXP sql_escape    p_sql_escape;
+extern DLL_IMP_EXP sql_query_func     p_sql_query;
+extern DLL_IMP_EXP sql_escape_func    p_sql_escape;
 
 extern DLL_IMP_EXP int           debug_level;
 extern DLL_IMP_EXP bool          dbg_timestamp;          /* print timestamp in debug output */
