@@ -37,6 +37,8 @@
 #define PRIO_SD_DEV_ACQUIRE   4            /* dev.acquire_mutex */
 #define PRIO_SD_DEV_ACCESS    5            /* dev.m_mutex */
 #define PRIO_SD_VOL_LIST      10           /* vol_list_lock */
+#define PRIO_SD_READ_VOL_LIST 12           /* read_vol_list */
 #define PRIO_SD_DEV_SPOOL     14           /* dev.spool_mutex */
+#define PRIO_SD_ACH_ACCESS    16           /* autochanger lock mutex */
 
 #endif
