@@ -1,7 +1,7 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -20,7 +20,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   Bacula(R) is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
@@ -40,7 +40,7 @@
  ****************************************************************************/
 
 #ifndef BUILD_PLUGIN
-extern DLL_IMP_EXP alist *plugin_list;
+extern DLL_IMP_EXP alist *bplugin_list;
 #endif
 
 /* Universal return codes from all plugin functions */
@@ -51,6 +51,9 @@ typedef enum {
   bRC_More   = 3,                        /* More files to backup */
   bRC_Term   = 4,                        /* Unload me */
   bRC_Seen   = 5,                        /* Return code from checkFiles */
+  bRC_Core   = 6,                        /* Let Bacula core handles this file */
+  bRC_Skip   = 7,                        /* Skip the proposed file */
+  bRC_Cancel = 8,                        /* Job cancelled */
 
   bRC_Max    = 9999                      /* Max code Bacula can use */
 } bRC;
@@ -71,11 +74,14 @@ typedef bRC (*t_unloadPlugin)(void);
 class Plugin {
 public:
    char *file;
+   int32_t file_len;
    t_unloadPlugin unloadPlugin;
    void *pinfo;
    void *pfuncs;
    void *pHandle;
    bool disabled;
+   bool restoreFileStarted;
+   bool createFileCalled;
 };
 
 /* Functions */
