@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -188,6 +188,9 @@ void jobstatus_to_ascii(int JobStatus, char *msg, int maxlen)
    case JS_Terminated:
       jobstat = _("OK");
       break;
+   case JS_Incomplete:
+      jobstat = _("Error: incomplete job");
+      break;
    case JS_FatalError:
    case JS_ErrorTerminated:
       jobstat = _("Error");
@@ -397,7 +400,7 @@ const char *job_type_to_str(int type)
 
 /* Convert ActionOnPurge to string (Truncate, Erase, Destroy)
  */
-char *aop_to_str(int aop, POOL_MEM &ret)
+char *action_on_purge_to_string(int aop, POOL_MEM &ret)
 {
    if (aop & ON_PURGE_TRUNCATE) {
       pm_strcpy(ret, _("Truncate"));
@@ -418,6 +421,7 @@ const char *job_level_to_str(int level)
    switch (level) {
    case L_BASE:
       str = _("Base");
+      break;
    case L_FULL:
       str = _("Full");
       break;
@@ -719,7 +723,7 @@ void decode_session_key(char *decode, char *session, char *key, int maxlen)
  *  %r = Recipients
  *  %v = Volume name
  *  %b = Job Bytes
- *  %f = Job Files
+ *  %F = Job Files
  *
  *  omsg = edited output message
  *  imsg = input string containing edit codes (%x)
@@ -805,7 +809,7 @@ POOLMEM *edit_job_codes(JCR *jcr, char *omsg, char *imsg, const char *to, job_co
                str = _("*none*");
             }
             break;
-         case 'f':                    /* Job Files */
+         case 'F':                    /* Job Files */
             str = edit_uint64(jcr->JobFiles, add);
             break;
          case 'b':                    /* Job Bytes */
