@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2007-2007 Free Software Foundation Europe e.V.
+   Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -37,7 +37,7 @@
 #include "label.h"
 #include <QMessageBox>
 
-labelPage::labelPage()
+labelPage::labelPage() : Pages()
 {
    QString deflt("");
    m_closeable = false;
@@ -48,7 +48,7 @@ labelPage::labelPage()
  * An overload of the constructor to have a default storage show in the
  * combobox on start.  Used from context sensitive in storage class.
  */
-labelPage::labelPage(QString &defString)
+labelPage::labelPage(QString &defString) : Pages()
 {
    showPage(defString);
 }
@@ -96,11 +96,15 @@ void labelPage::okButtonPushed()
                   .arg(storageCombo->currentText()) 
                   .arg(slotSpin->value());
    if (mainWin->m_commandDebug) {
-      Pmsg1(000, "sending command : %s\n",scmd.toUtf8().data());
+      Pmsg1(000, "sending command : %s\n", scmd.toUtf8().data());
+   }
+   if (m_console) {
+      m_console->write_dir(scmd.toUtf8().data());
+      m_console->displayToPrompt(m_conn);
+      m_console->notify(m_conn, true);
+   } else {
+      Pmsg0(000, "m_console==NULL !!!!!!\n");
    }
-   m_console->write_dir(scmd.toUtf8().data());
-   m_console->displayToPrompt(m_conn);
-   m_console->notify(m_conn, true);
    closeStackPage();
    mainWin->resetFocus();
 }
@@ -108,7 +112,11 @@ void labelPage::okButtonPushed()
 void labelPage::cancelButtonPushed()
 {
    this->hide();
-   m_console->notify(m_conn, true);
+   if (m_console) {
+      m_console->notify(m_conn, true);
+   } else {
+      Pmsg0(000, "m_console==NULL !!!!!!\n");
+   }
    closeStackPage();
    mainWin->resetFocus();
 }
