@@ -4,7 +4,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2007-2007 Free Software Foundation Europe e.V.
+   Copyright (C) 2007-2010 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -29,13 +29,34 @@
    Switzerland, email:ftf@fsfeurope.org.
 */
 /*
- *   Version $Id$
  *
  *  Kern Sibbald, February 2007
  */
 
+#include <sys/types.h>
+
 #include <QtGui>
 #include "pages.h"
+#include "ui_runrestore.h"
+
+class bRestoreTable : public QTableWidget
+{
+   Q_OBJECT 
+private:
+   QPoint dragStartPosition;
+public:
+   bRestoreTable(QWidget *parent)
+      : QTableWidget(parent)
+   {
+   }
+   void mousePressEvent(QMouseEvent *event);
+   void mouseMoveEvent(QMouseEvent *event);
+
+   void dragEnterEvent(QDragEnterEvent *event);
+   void dragMoveEvent(QDragMoveEvent *event);
+   void dropEvent(QDropEvent *event);
+};
+
 #include "ui_brestore.h"
 #include "ui_restore.h"
 #include "ui_prerestore.h"
@@ -113,7 +134,6 @@ private:
    QString m_splitText;
 };
 
-
 class bRestore : public Pages, public Ui::bRestoreForm
 {
    Q_OBJECT 
@@ -121,11 +141,49 @@ class bRestore : public Pages, public Ui::bRestoreForm
 public:
    bRestore();
    ~bRestore();
+   void PgSeltreeWidgetClicked();
+   QString m_client;
+   QString m_jobids;
+   void get_info_from_selection(QStringList &fileids, QStringList &jobids,
+                                QStringList &dirids, QStringList &fileindexes);
 
 public slots:
-
+   void setClient();
+   void setJob();
+   void showInfoForFile(QTableWidgetItem *);
+   void applyLocation();
+   void clearVersions(QTableWidgetItem *);
+   void clearRestoreList();
+   void runRestore();
+   void refreshView();
 private:
+   QString m_path;
+   int64_t m_pathid;
+   QTableWidgetItem *m_current;
+   void setupPage();
+   bool m_populated;
+   void displayFiles(int64_t pathid, QString path);
+   void displayFileVersion(QString pathid, QString fnid, 
+                           QString client, QString filename);
+};
 
+class bRunRestore : public QDialog, public Ui::bRunRestoreForm
+{
+   Q_OBJECT 
+private:
+   bRestore *brestore;
+   QStringList m_fileids, m_jobids, m_dirids, m_findexes;
+
+public:
+   bRunRestore(bRestore *parent);
+   ~bRunRestore() {}
+   void computeVolumeList();
+   int64_t runRestore(QString tablename);
+
+public slots:
+   void useRegexp();
+   void UFRcb();
+   void computeRestore();
 };
 
 #endif /* _RESTORE_H_ */
