@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2007-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2007-2010 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -31,7 +31,6 @@
  *
  *   Kern Sibbald, March MMVII
  *
- *  $Id: select.cpp 8775 2009-04-30 16:57:18Z bartleyd2 $
  */ 
 
 #include "bat.h"
@@ -46,9 +45,11 @@ textInputDialog::textInputDialog(Console *console, int conn)
    QDateTime dt;
 
    m_console = console;
+   m_console->notify(m_conn, false);
    setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);
-   labelWidget->setText(m_console->returnFromPrompt(m_conn));
+   m_console->read(m_conn);                 /* get title */
+   labelWidget->setText(m_console->msg(m_conn));
    this->show();
 }
 
@@ -56,11 +57,10 @@ void textInputDialog::accept()
 {
    this->hide();
    m_console->write_dir(m_conn, lineEdit->text().toUtf8().data());
-   m_console->displayToPrompt(m_conn);
-   mainWin->resetFocus();
-   m_console->displayToPrompt(m_conn);
-   m_console->notify(m_conn, true);
+   /* Do not displayToPrompt because there may be another Text Input required */
    this->close();
+   mainWin->resetFocus();
+   m_console->notify(m_conn, true);
 }
 
 
@@ -68,9 +68,9 @@ void textInputDialog::reject()
 {
    this->hide();
    mainWin->set_status(tr(" Canceled"));
+   m_console->write_dir(m_conn, ".");
+   this->close();
    mainWin->resetFocus();
    m_console->beginNewCommand(m_conn);
    m_console->notify(m_conn, true);
-   this->close();
 }
-
