@@ -92,8 +92,8 @@ JCR *setup_jcr(const char *name, char *dev_name, BSR *bsr,
    jcr->NumReadVolumes = 0;
    jcr->NumWriteVolumes = 0;
    jcr->JobId = 0;
-   jcr->set_JobType(JT_CONSOLE);
-   jcr->set_JobLevel(L_FULL);
+   jcr->setJobType(JT_CONSOLE);
+   jcr->setJobLevel(L_FULL);
    jcr->JobStatus = JS_Terminated;
    jcr->where = bstrdup("");
    jcr->job_name = get_pool_memory(PM_FNAME);
@@ -105,6 +105,8 @@ JCR *setup_jcr(const char *name, char *dev_name, BSR *bsr,
    pm_strcpy(jcr->fileset_name, "Dummy.fileset.name");
    jcr->fileset_md5 = get_pool_memory(PM_FNAME);
    pm_strcpy(jcr->fileset_md5, "Dummy.fileset.md5");
+   jcr->comment = get_pool_memory(PM_MESSAGE);
+   *jcr->comment = '\0';
    init_autochangers();
    create_volume_lists();
 
@@ -221,6 +223,10 @@ static void my_free_jcr(JCR *jcr)
       free_pool_memory(jcr->fileset_md5);
       jcr->fileset_md5 = NULL;
    }
+   if (jcr->comment) {
+      free_pool_memory(jcr->comment);
+      jcr->comment = NULL;
+   }
    if (jcr->VolList) {
       free_restore_volume_list(jcr);
    }
