@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2002-2008 Free Software Foundation Europe e.V.
+   Copyright (C) 2002-2010 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -38,7 +38,6 @@
  *
  *    Kern E. Sibbald, August MMII
  *
- *   Version $Id$
  */
 
 #include "bacula.h"
@@ -53,6 +52,7 @@ static char *rec_state_to_str(DEV_RECORD *rec);
 #endif
 
 static const int dbglvl = 500;
+static const int no_FileIndex = -999999;
 
 /*
  * This subroutine reads all the records and passes them back to your
@@ -68,6 +68,7 @@ bool read_records(DCR *dcr,
    DEV_BLOCK *block = dcr->block;
    DEV_RECORD *rec = NULL;
    uint32_t record;
+   int32_t lastFileIndex;
    bool ok = true;
    bool done = false;
    SESSION_LABEL sessrec;
@@ -195,6 +196,7 @@ bool read_records(DCR *dcr,
             block->BlockNumber, rec->remainder);
       record = 0;
       rec->state = 0;
+      lastFileIndex = no_FileIndex;
       Dmsg1(dbglvl, "Block %s empty\n", is_block_empty(rec)?"is":"NOT");
       for (rec->state=0; ok && !is_block_empty(rec); ) {
          if (!read_record_from_block(dcr, block, rec)) {
@@ -284,22 +286,35 @@ bool read_records(DCR *dcr,
          Dmsg6(dbglvl, "OK callback. recno=%d state=%s blk=%d SI=%d ST=%d FI=%d\n", record,
                rec_state_to_str(rec), block->BlockNumber,
                rec->VolSessionId, rec->VolSessionTime, rec->FileIndex);
+         if (lastFileIndex != no_FileIndex && lastFileIndex != rec->FileIndex) {
+            if (is_this_bsr_done(jcr->bsr, rec) && try_repositioning(jcr, rec, dcr)) {
+               Dmsg2(dbglvl, "This bsr done, break pos %u:%u\n",
+                     dev->file, dev->block_num);
+               break;
+            }
+            Dmsg2(dbglvl, "==== inside LastIndex=%d FileIndex=%d\n", lastFileIndex, rec->FileIndex);
+         }
+         Dmsg2(dbglvl, "==== LastIndex=%d FileIndex=%d\n", lastFileIndex, rec->FileIndex);
+         lastFileIndex = rec->FileIndex;
          ok = record_cb(dcr, rec);
+#if 0
          /*
           * If we have a digest stream, we check to see if we have 
           *  finished the current bsr, and if so, repositioning will
           *  be turned on.
           */
          if (crypto_digest_stream_type(rec->Stream) != CRYPTO_DIGEST_NONE) {
-            Dmsg3(dbglvl, "Have digest FI=%u before bsr check pos %u:%u\n", rec->FileIndex,
+            Dmsg3(dbglvl, "=== Have digest FI=%u before bsr check pos %u:%u\n", rec->FileIndex,
                   dev->file, dev->block_num);
             if (is_this_bsr_done(jcr->bsr, rec) && try_repositioning(jcr, rec, dcr)) {
+               Dmsg1(dbglvl, "==== BSR done at FI=%d\n", rec->FileIndex);
                Dmsg2(dbglvl, "This bsr done, break pos %u:%u\n",
                      dev->file, dev->block_num);
                break;
             }
             Dmsg2(900, "After is_bsr_done pos %u:%u\n", dev->file, dev->block_num);
          }
+#endif
       } /* end for loop over records */
       Dmsg2(dbglvl, "After end recs in block. pos=%u:%u\n", dev->file, dev->block_num);
    } /* end for loop over blocks */
@@ -393,7 +408,7 @@ static void handle_session_record(DEVICE *dev, DEV_RECORD *rec, SESSION_LABEL *s
    const char *rtype;
    char buf[100];
 
-   memset(sessrec, 0, sizeof(sessrec));
+   memset(sessrec, 0, sizeof(SESSION_LABEL));
    switch (rec->FileIndex) {
    case PRE_LABEL:
       rtype = _("Fresh Volume Label");
