@@ -226,9 +226,9 @@ static s_kw dev_types[] = {
  */
 static void store_devtype(LEX *lc, RES_ITEM *item, int index, int pass)
 {
-   int token, i;
+   int i;
 
-   token = lex_get_token(lc, T_NAME);
+   lex_get_token(lc, T_NAME);
    /* Store the label pass 2 so that type is defined */
    for (i=0; dev_types[i].name; i++) {
       if (strcasecmp(lc->str, dev_types[i].name) == 0) {
@@ -457,6 +457,7 @@ void free_resource(RES *sres, int type)
       if (res->res_changer.device) {
          delete res->res_changer.device;
       }
+      rwl_destroy(&res->res_changer.changer_lock);
       break; 
    case R_STORAGE:
       if (res->res_store.sdaddrs) {
@@ -474,6 +475,9 @@ void free_resource(RES *sres, int type)
       if (res->res_store.subsys_directory) {
          free(res->res_store.subsys_directory);
       }
+      if (res->res_store.plugin_directory) {
+         free(res->res_store.plugin_directory);
+      }
       if (res->res_store.scripts_directory) {
          free(res->res_store.scripts_directory);
       }
@@ -629,9 +633,11 @@ void save_resource(int type, RES_ITEM *items, int pass)
          foreach_alist(dev, res->res_changer.device) {
             dev->changer_res = (AUTOCHANGER *)&res->res_changer;
          }
-         if ((errstat = pthread_mutex_init(&res->res_changer.changer_mutex, NULL)) != 0) {
+         if ((errstat = rwl_init(&res->res_changer.changer_lock, 
+                                 PRIO_SD_ACH_ACCESS)) != 0)
+         {
             berrno be;
-            Jmsg1(NULL, M_ERROR_TERM, 0, _("Unable to init mutex: ERR=%s\n"), 
+            Jmsg1(NULL, M_ERROR_TERM, 0, _("Unable to init lock: ERR=%s\n"), 
                   be.bstrerror(errstat));
          }
          break;
