@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2002-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2002-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -32,19 +32,20 @@
  *
  *   Kern E. Sibbald, August 2002
  *
- *   Version $Id$
- *
  */
 
 #include "bacula.h"
 #include "cats/cats.h"
+#include "cats/sql_glue.h"
 #include "lib/runscript.h"
 #include "dird/dird_conf.h"
 
 extern bool parse_dir_config(CONFIG *config, const char *configfile, int exit_code);
 
-/* Dummy functions */
-int generate_daemon_event(JCR *jcr, const char *event) 
+/*
+ * Dummy functions
+ */
+int generate_daemon_event(JCR *jcr, const char *event)
    { return 1; }
 
 typedef struct s_id_ctx {
@@ -63,7 +64,9 @@ typedef struct s_name_ctx {
    int tot_ids;                       /* total to process */
 } NAME_LIST;
 
-/* Global variables */
+/*
+ * Global variables
+ */
 static bool fix = false;
 static bool batch = false;
 static B_DB *db;
@@ -72,10 +75,14 @@ static NAME_LIST name_list;
 static char buf[20000];
 static bool quit = false;
 static CONFIG *config;
+static const char *idx_tmp_name;
 
 #define MAX_ID_LIST_LEN 10000000
 
-/* Forward referenced functions */
+/*
+ * Forward referenced functions
+ */
+static void print_catalog_details(CAT *catalog, const char *working_dir);
 static int make_id_list(const char *query, ID_LIST *id_list);
 static int delete_id_list(const char *query, ID_LIST *id_list);
 static int make_name_list(const char *query, NAME_LIST *name_list);
@@ -98,16 +105,10 @@ static void repair_bad_filenames();
 static void do_interactive_mode();
 static bool yes_no(const char *prompt);
 static bool check_idx(const char *col_name);
-static bool create_tmp_idx(const char *idx_name, const char *table_name, 
+static bool create_tmp_idx(const char *idx_name, const char *table_name,
               const char *col_name);
 static bool drop_tmp_idx(const char *idx_name, const char *table_name);
-#ifdef HAVE_MYSQL
 static int check_idx_handler(void *ctx, int num_fields, char **row);
-#endif
-
-
-/* Global variables */
-static const char *idx_tmp_name;
 
 static void usage()
 {
@@ -131,7 +132,6 @@ int main (int argc, char *argv[])
    int ch;
    const char *user, *password, *db_name, *dbhost;
    int dbport = 0;
-   bool test_thread=false;
    bool print_catalog=false;
    char *configfile = NULL;
    char *catalogname = NULL;
@@ -143,29 +143,25 @@ int main (int argc, char *argv[])
    lmgr_init_thread();
 
    my_name_is(argc, argv, "dbcheck");
-   init_msg(NULL, NULL);              /* setup message handler */
+   init_msg(NULL, NULL);           /* setup message handler */
 
    memset(&id_list, 0, sizeof(id_list));
    memset(&name_list, 0, sizeof(name_list));
 
-   while ((ch = getopt(argc, argv, "bc:C:d:fvBt?")) != -1) {
+   while ((ch = getopt(argc, argv, "bc:C:d:fvB?")) != -1) {
       switch (ch) {
       case 'B':
          print_catalog = true;     /* get catalog information from config */
          break;
-
       case 'b':                    /* batch */
          batch = true;
          break;
-
       case 'C':                    /* CatalogName */
           catalogname = optarg;
          break;
-
       case 'c':                    /* configfile */
           configfile = optarg;
          break;
-
       case 'd':                    /* debug level */
          if (*optarg == 't') {
             dbg_timestamp = true;
@@ -176,18 +172,12 @@ int main (int argc, char *argv[])
             }
          }
          break;
-
       case 'f':                    /* fix inconsistencies */
          fix = true;
          break;
-
       case 'v':
          verbose++;
          break;
-      case 't':
-         test_thread=true;
-         break;
-
       case '?':
       default:
          usage();
@@ -198,16 +188,6 @@ int main (int argc, char *argv[])
 
    OSDependentInit();
 
-   if (test_thread) {
-      /* When we will load the SQL backend with ldopen, this check would be
-       * moved after the database initialization. It will need a valid config
-       * file.
-       */
-      db_check_backend_thread_safe();
-      Pmsg0(0, _("OK - DB backend seems to be thread-safe.\n"));
-      exit(0);
-   }
-
    if (configfile) {
       CAT *catalog = NULL;
       int found = 0;
@@ -245,12 +225,11 @@ int main (int argc, char *argv[])
          }
          set_working_directory(director->working_directory);
 
-         /* Print catalog information and exit (-B) */
+         /*
+          * Print catalog information and exit (-B)
+          */
          if (print_catalog) {
-            POOLMEM *buf = get_pool_memory(PM_MESSAGE);
-            printf("%sdb_type=%s\nworking_dir=%s\n", catalog->display(buf),
-                   db_get_type(), working_directory);
-            free_pool_memory(buf);
+            print_catalog_details(catalog, director->working_directory);
             exit(0);
          }
 
@@ -274,7 +253,9 @@ int main (int argc, char *argv[])
          usage();
       }
 
-      /* This is needed by SQLite to find the db */
+      /*
+       * This is needed by SQLite to find the db
+       */
       working_directory = argv[0];
       db_name = "bacula";
       user = db_name;
@@ -313,13 +294,20 @@ int main (int argc, char *argv[])
       }
    }
 
-   /* Open database */
-   db = db_init_database(NULL, db_name, user, password, dbhost, dbport, NULL, 0);
+   /*
+    * Open database
+    */
+   db = db_init_database(NULL, NULL, db_name, user, password, dbhost, dbport, NULL, false, false);
    if (!db_open_database(NULL, db)) {
       Emsg1(M_FATAL, 0, "%s", db_strerror(db));
           return 1;
    }
 
+   /*
+    * Drop temporary index idx_tmp_name if it already exists
+    */
+   drop_tmp_idx("idxPIchk", "File");
+
    if (batch) {
       repair_bad_paths();
       repair_bad_filenames();
@@ -338,6 +326,11 @@ int main (int argc, char *argv[])
       do_interactive_mode();
    }
 
+   /*
+    * Drop temporary index idx_tmp_name
+    */
+   drop_tmp_idx("idxPIchk", "File");
+
    db_close_database(NULL, db);
    close_msg(NULL);
    term_msg();
@@ -345,6 +338,26 @@ int main (int argc, char *argv[])
    return 0;
 }
 
+static void print_catalog_details(CAT *catalog, const char *working_dir)
+{
+   POOLMEM *catalog_details = get_pool_memory(PM_MESSAGE);
+
+   /*
+    * Instantiate a B_DB class and see what db_type gets assigned to it.
+    */
+   db = db_init_database(NULL, catalog->db_driver, catalog->db_name, catalog->db_user,
+                         catalog->db_password, catalog->db_address,
+                         catalog->db_port, catalog->db_socket,
+                         catalog->mult_db_connections,
+                         catalog->disable_batch_insert);
+   if (db) {
+      printf("%sdb_type=%s\nworking_dir=%s\n", catalog->display(catalog_details),
+             db->db_get_type(), working_directory);
+      db_close_database(NULL, db);
+   }
+   free_pool_memory(catalog_details);
+}
+
 static void do_interactive_mode()
 {
    const char *cmd;
@@ -492,9 +505,10 @@ static int print_name_handler(void *ctx, int num_fields, char **row)
 
 static int get_name_handler(void *ctx, int num_fields, char **row)
 {
-   POOLMEM *buf = (POOLMEM *)ctx;
+   POOLMEM *name = (POOLMEM *)ctx;
+
    if (row[0]) {
-      pm_strcpy(&buf, row[0]);
+      pm_strcpy(&name, row[0]);
    }
    return 0;
 }
@@ -506,7 +520,6 @@ static int print_job_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-
 static int print_jobmedia_handler(void *ctx, int num_fields, char **row)
 {
    printf(_("Orphaned JobMediaId=%s JobId=%s Volume=\"%s\"\n"),
@@ -535,7 +548,6 @@ static int print_client_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-
 /*
  * Called here with each id to be added to the list
  */
@@ -614,7 +626,6 @@ static int name_list_handler(void *ctx, int num_fields, char **row)
    return 0;
 }
 
-
 /*
  * Construct name list
  */
@@ -641,7 +652,6 @@ static void print_name_list(NAME_LIST *name_list)
    }
 }
 
-
 /*
  * Free names in the list
  */
@@ -660,7 +670,9 @@ static void eliminate_duplicate_filenames()
 
    printf(_("Checking for duplicate Filename entries.\n"));
 
-   /* Make list of duplicated names */
+   /*
+    * Make list of duplicated names
+    */
    query = "SELECT Name, count(Name) as Count FROM Filename GROUP BY  Name "
            "HAVING count(Name) > 1";
 
@@ -675,9 +687,13 @@ static void eliminate_duplicate_filenames()
       return;
    }
    if (fix) {
-      /* Loop through list of duplicate names */
+      /*
+       * Loop through list of duplicate names
+       */
       for (int i=0; i<name_list.num_ids; i++) {
-         /* Get all the Ids of each name */
+         /*
+          * Get all the Ids of each name
+          */
          db_escape_string(NULL, db, esc_name, name_list.name[i], strlen(name_list.name[i]));
          bsnprintf(buf, sizeof(buf), "SELECT FilenameId FROM Filename WHERE Name='%s'", esc_name);
          if (verbose > 1) {
@@ -689,7 +705,9 @@ static void eliminate_duplicate_filenames()
          if (verbose) {
             printf(_("Found %d for: %s\n"), id_list.num_ids, name_list.name[i]);
          }
-         /* Force all records to use the first id then delete the other ids */
+         /*
+          * Force all records to use the first id then delete the other ids
+          */
          for (int j=1; j<id_list.num_ids; j++) {
             char ed1[50], ed2[50];
             bsnprintf(buf, sizeof(buf), "UPDATE File SET FilenameId=%s WHERE FilenameId=%s",
@@ -717,8 +735,9 @@ static void eliminate_duplicate_paths()
 
    printf(_("Checking for duplicate Path entries.\n"));
 
-   /* Make list of duplicated names */
-
+   /*
+    * Make list of duplicated names
+    */
    query = "SELECT Path, count(Path) as Count FROM Path "
            "GROUP BY Path HAVING count(Path) > 1";
 
@@ -733,9 +752,13 @@ static void eliminate_duplicate_paths()
       return;
    }
    if (fix) {
-      /* Loop through list of duplicate names */
+      /*
+       * Loop through list of duplicate names
+       */
       for (int i=0; i<name_list.num_ids; i++) {
-         /* Get all the Ids of each name */
+         /*
+          * Get all the Ids of each name
+          */
          db_escape_string(NULL, db, esc_name, name_list.name[i], strlen(name_list.name[i]));
          bsnprintf(buf, sizeof(buf), "SELECT PathId FROM Path WHERE Path='%s'", esc_name);
          if (verbose > 1) {
@@ -747,7 +770,9 @@ static void eliminate_duplicate_paths()
          if (verbose) {
             printf(_("Found %d for: %s\n"), id_list.num_ids, name_list.name[i]);
          }
-         /* Force all records to use the first id then delete the other ids */
+         /*
+          * Force all records to use the first id then delete the other ids
+          */
          for (int j=1; j<id_list.num_ids; j++) {
             char ed1[50], ed2[50];
             bsnprintf(buf, sizeof(buf), "UPDATE File SET PathId=%s WHERE PathId=%s",
@@ -777,7 +802,9 @@ static void eliminate_orphaned_jobmedia_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /* Loop doing 300000 at a time */
+   /*
+    * Loop doing 300000 at a time
+    */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned JobMedia records.\n"), id_list.num_ids);
       if (id_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
@@ -785,7 +812,7 @@ static void eliminate_orphaned_jobmedia_records()
             char ed1[50];
             bsnprintf(buf, sizeof(buf),
 "SELECT JobMedia.JobMediaId,JobMedia.JobId,Media.VolumeName FROM JobMedia,Media "
-"WHERE JobMedia.JobMediaId=%s AND Media.MediaId=JobMedia.MediaId", 
+"WHERE JobMedia.JobMediaId=%s AND Media.MediaId=JobMedia.MediaId",
                edit_int64(id_list.Id[i], ed1));
             if (!db_sql_query(db, buf, print_jobmedia_handler, NULL)) {
                printf("%s\n", db_strerror(db));
@@ -821,7 +848,9 @@ static void eliminate_orphaned_file_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /* Loop doing 300000 at a time */
+   /*
+    * Loop doing 300000 at a time
+    */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned File records.\n"), id_list.num_ids);
       if (name_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
@@ -829,7 +858,7 @@ static void eliminate_orphaned_file_records()
             char ed1[50];
             bsnprintf(buf, sizeof(buf),
 "SELECT File.FileId,File.JobId,Filename.Name FROM File,Filename "
-"WHERE File.FileId=%s AND File.FilenameId=Filename.FilenameId", 
+"WHERE File.FileId=%s AND File.FilenameId=Filename.FilenameId",
                edit_int64(id_list.Id[i], ed1));
             if (!db_sql_query(db, buf, print_file_handler, NULL)) {
                printf("%s\n", db_strerror(db));
@@ -853,11 +882,25 @@ static void eliminate_orphaned_file_records()
 
 static void eliminate_orphaned_path_records()
 {
+   db_int64_ctx lctx;
+   lctx.count=0;
+   db_sql_query(db, "SELECT 1 FROM Job WHERE HasCache=1 LIMIT 1", 
+                db_int64_handler, &lctx);
+   
+   if (lctx.count == 1) {
+      printf(_("Pruning orphaned Path entries isn't possible when using BVFS.\n"));
+      return;
+   }
+
    idx_tmp_name = NULL;
-   /* check the existence of the required "one column" index */
+   /*
+    * Check the existence of the required "one column" index
+    */
    if (!check_idx("PathId"))  {
       if (yes_no(_("Create temporary index? (yes/no): "))) {
-         /* create temporary index PathId  */
+         /*
+          * create temporary index PathId
+          */
          create_tmp_idx("idxPIchk", "File", "PathId");
       }
    }
@@ -873,13 +916,15 @@ static void eliminate_orphaned_path_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /* Loop doing 300000 at a time */
+   /*
+    * Loop doing 300000 at a time
+    */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned Path records.\n"), id_list.num_ids);
       if (id_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
          for (int i=0; i < id_list.num_ids; i++) {
             char ed1[50];
-            bsnprintf(buf, sizeof(buf), "SELECT Path FROM Path WHERE PathId=%s", 
+            bsnprintf(buf, sizeof(buf), "SELECT Path FROM Path WHERE PathId=%s",
                edit_int64(id_list.Id[i], ed1));
             db_sql_query(db, buf, print_name_handler, NULL);
          }
@@ -896,18 +941,24 @@ static void eliminate_orphaned_path_records()
       if (!make_id_list(query, &id_list)) {
          exit(1);
       }
-   } 
-   /* drop temporary index idx_tmp_name */
+   }
+   /*
+    * Drop temporary index idx_tmp_name
+    */
    drop_tmp_idx("idxPIchk", "File");
 }
 
 static void eliminate_orphaned_filename_records()
 {
    idx_tmp_name = NULL;
-   /* check the existence of the required "one column" index */
+   /*
+    * Check the existence of the required "one column" index
+    */
    if (!check_idx("FilenameId") )      {
       if (yes_no(_("Create temporary index? (yes/no): "))) {
-         /* create temporary index FilenameId  */
+         /*
+          * Create temporary index FilenameId
+          */
          create_tmp_idx("idxFIchk", "File", "FilenameId");
       }
    }
@@ -923,13 +974,15 @@ static void eliminate_orphaned_filename_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /* Loop doing 300000 at a time */
+   /*
+    * Loop doing 300000 at a time
+    */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned Filename records.\n"), id_list.num_ids);
       if (id_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
          for (int i=0; i < id_list.num_ids; i++) {
             char ed1[50];
-            bsnprintf(buf, sizeof(buf), "SELECT Name FROM Filename WHERE FilenameId=%s", 
+            bsnprintf(buf, sizeof(buf), "SELECT Name FROM Filename WHERE FilenameId=%s",
                edit_int64(id_list.Id[i], ed1));
             db_sql_query(db, buf, print_name_handler, NULL);
          }
@@ -947,7 +1000,9 @@ static void eliminate_orphaned_filename_records()
          exit(1);
       }
    }
-   /* drop temporary index idx_tmp_name */
+   /*
+    * Drop temporary index idx_tmp_name
+    */
    drop_tmp_idx("idxFIchk", "File");
 
 }
@@ -991,7 +1046,8 @@ static void eliminate_orphaned_client_records()
    const char *query;
 
    printf(_("Checking for orphaned Client entries.\n"));
-   /* In English:
+   /*
+    * In English:
     *   Wiffle through Client for every Client
     *   joining with the Job table including every Client even if
     *   there is not a match in Job (left outer join), then
@@ -1032,7 +1088,8 @@ static void eliminate_orphaned_job_records()
    const char *query;
 
    printf(_("Checking for orphaned Job entries.\n"));
-   /* In English:
+   /*
+    * In English:
     *   Wiffle through Job for every Job
     *   joining with the Client table including every Job even if
     *   there is not a match in Client (left outer join), then
@@ -1072,7 +1129,6 @@ static void eliminate_orphaned_job_records()
    }
 }
 
-
 static void eliminate_admin_records()
 {
    const char *query;
@@ -1139,9 +1195,6 @@ static void eliminate_restore_records()
    }
 }
 
-
-
-
 static void repair_bad_filenames()
 {
    const char *query;
@@ -1161,7 +1214,7 @@ static void repair_bad_filenames()
       for (i=0; i < id_list.num_ids; i++) {
          char ed1[50];
          bsnprintf(buf, sizeof(buf),
-            "SELECT Name FROM Filename WHERE FilenameId=%s", 
+            "SELECT Name FROM Filename WHERE FilenameId=%s",
                 edit_int64(id_list.Id[i], ed1));
          if (!db_sql_query(db, buf, print_name_handler, NULL)) {
             printf("%s\n", db_strerror(db));
@@ -1179,12 +1232,14 @@ static void repair_bad_filenames()
          int len;
          char ed1[50];
          bsnprintf(buf, sizeof(buf),
-            "SELECT Name FROM Filename WHERE FilenameId=%s", 
+            "SELECT Name FROM Filename WHERE FilenameId=%s",
                edit_int64(id_list.Id[i], ed1));
          if (!db_sql_query(db, buf, get_name_handler, name)) {
             printf("%s\n", db_strerror(db));
          }
-         /* Strip trailing slash(es) */
+         /*
+          * Strip trailing slash(es)
+          */
          for (len=strlen(name); len > 0 && IsPathSeparator(name[len-1]); len--)
             {  }
          if (len == 0) {
@@ -1203,6 +1258,7 @@ static void repair_bad_filenames()
          }
          db_sql_query(db, buf, NULL, NULL);
       }
+      free_pool_memory(name);
    }
 }
 
@@ -1246,11 +1302,15 @@ static void repair_bad_paths()
          if (!db_sql_query(db, buf, get_name_handler, name)) {
             printf("%s\n", db_strerror(db));
          }
-         /* Strip trailing blanks */
+         /*
+          * Strip trailing blanks
+          */
          for (len=strlen(name); len > 0 && name[len-1]==' '; len--) {
             name[len-1] = 0;
          }
-         /* Add trailing slash */
+         /*
+          * Add trailing slash
+          */
          len = pm_strcat(&name, "/");
          db_escape_string(NULL, db, esc_name, name, len);
          bsnprintf(buf, sizeof(buf), "UPDATE Path SET Path='%s' WHERE PathId=%s",
@@ -1260,10 +1320,10 @@ static void repair_bad_paths()
          }
          db_sql_query(db, buf, NULL, NULL);
       }
+      free_pool_memory(name);
    }
 }
 
-
 /*
  * Gen next input command from the terminal
  */
@@ -1294,13 +1354,11 @@ static bool yes_no(const char *prompt)
 
 bool python_set_prog(JCR*, char const*) { return false; }
 
-
 /*
  * The code below to add indexes is needed only for MySQL, and
  *  that to improve the performance.
  */
 
-#ifdef HAVE_MYSQL
 #define MAXIDX          100
 typedef struct s_idx_list {
    char *key_name;
@@ -1310,17 +1368,19 @@ typedef struct s_idx_list {
 
 static IDX_LIST idx_list[MAXIDX];
 
-/* 
+/*
  * Called here with each table index to be added to the list
  */
 static int check_idx_handler(void *ctx, int num_fields, char **row)
 {
-   /* Table | Non_unique | Key_name | Seq_in_index | Column_name |...
-    * File  |          0 | PRIMARY  |            1 | FileId      |... 
+   /*
+    * Table | Non_unique | Key_name | Seq_in_index | Column_name |...
+    * File  |          0 | PRIMARY  |            1 | FileId      |...
     */
    char *name, *key_name, *col_name;
    int i, len;
    int found = false;
+
    name = (char *)ctx;
    key_name = row[2];
    col_name = row[4];
@@ -1334,7 +1394,9 @@ static int check_idx_handler(void *ctx, int num_fields, char **row)
          break;
       }
    }
-   /* if the new Key_name, add it to the list */
+   /*
+    * If the new Key_name, add it to the list
+    */
    if (!found) {
       len = strlen(key_name) + 1;
       idx_list[i].key_name = (char *)malloc(len);
@@ -1348,53 +1410,55 @@ static int check_idx_handler(void *ctx, int num_fields, char **row)
    }
    return 0;
 }
-#endif
 
 /*
  * Return TRUE if "one column" index over *col_name exists
  */
 static bool check_idx(const char *col_name)
 {
-#ifdef HAVE_MYSQL
    int i;
    int found = false;
-
-   memset(&idx_list, 0, sizeof(idx_list));
    const char *query = "SHOW INDEX FROM File";
-   if (!db_sql_query(db, query, check_idx_handler, (void *)col_name)) {
-      printf("%s\n", db_strerror(db));
-   }
 
-   for(i = 0; (idx_list[i].key_name != NULL) && (i < MAXIDX) ; i++) {
-      /* NOTE : if (idx_list[i].count_key > 1) then index idx_list[i].key_name is "multiple-column" index */
-      if ((idx_list[i].count_key == 1) && (idx_list[i].count_col == 1)) {
-         /* "one column" index over *col_name found */
-         found = true;
+   switch (db_get_type_index(db)) {
+   case SQL_TYPE_MYSQL:
+      memset(&idx_list, 0, sizeof(idx_list));
+      if (!db_sql_query(db, query, check_idx_handler, (void *)col_name)) {
+         printf("%s\n", db_strerror(db));
       }
-   }
-   if (found) {
-      if (verbose) {
-         printf(_("Ok. Index over the %s column already exists and dbcheck will work faster.\n"), col_name);
+      for (i = 0; (idx_list[i].key_name != NULL) && (i < MAXIDX) ; i++) {
+         /*
+          * NOTE : if (idx_list[i].count_key > 1) then index idx_list[i].key_name is "multiple-column" index
+          */
+         if ((idx_list[i].count_key == 1) && (idx_list[i].count_col == 1)) {
+            /*
+             * "one column" index over *col_name found
+             */
+            found = true;
+         }
       }
-   } else {
-      printf(_("Note. Index over the %s column not found, that can greatly slow down dbcheck.\n"), col_name);
+      if (found) {
+         if (verbose) {
+            printf(_("Ok. Index over the %s column already exists and dbcheck will work faster.\n"), col_name);
+         }
+      } else {
+         printf(_("Note. Index over the %s column not found, that can greatly slow down dbcheck.\n"), col_name);
+      }
+      return found;
+   default:
+      return true;
    }
-
-   return found;
-#else
-   return true;
-#endif
 }
 
 /*
  * Create temporary one-column index
  */
-static bool create_tmp_idx(const char *idx_name, const char *table_name, 
-               const char *col_name)
+static bool create_tmp_idx(const char *idx_name, const char *table_name,
+                           const char *col_name)
 {
    idx_tmp_name = NULL;
    printf(_("Create temporary index... This may take some time!\n"));
-   bsnprintf(buf, sizeof(buf), "CREATE INDEX %s ON %s (%s)", idx_name, table_name, col_name); 
+   bsnprintf(buf, sizeof(buf), "CREATE INDEX %s ON %s (%s)", idx_name, table_name, col_name);
    if (verbose) {
       printf("%s\n", buf);
    }
