@@ -1,9 +1,3 @@
-/*
- * Define Host machine
- *
- *  Version $Id$
- *
- */
 /*
    Bacula® - The Network Backup Solution
 
@@ -32,16 +26,23 @@
    Switzerland, email:ftf@fsfeurope.org.
 */
 
+/*
+ * Define Host machine
+ */
+
+
 #include "host.h"
 #undef HOST_OS
 #undef DISTNAME
 #undef DISTVER
 
 #ifdef HAVE_MINGW
-
+extern char win_os[];
 #define HOST_OS  "Linux"
 #define DISTNAME "Cross-compile"
+#ifndef BACULA
 #define BACULA "Bacula"
+#endif
 #ifdef _WIN64
 # define DISTVER "Win64"
 #else
